"""进度显示工具

提供简洁的进度条显示和整体执行进度管理
"""

import sys
import time
from typing import Any, Callable, Optional, TextIO

from .utils import format_file_size


class ProgressBar:
    """统一的进度条显示"""

    def __init__(
        self, name: str = "上传", bar_width: int = 40, file: Optional[TextIO] = None
    ):
        self.name = name
        self.bar_width = bar_width
        self.file = file or sys.stderr
        self.start_time = time.time()
        self.last_update = 0

    def update(self, consumed: int, total: int):
        """更新进度条"""
        now = time.time()
        # 限制更新频率，避免闪烁
        if now - self.last_update < 0.1 and consumed < total:
            return

        percentage = min(100, (consumed / total) * 100) if total > 0 else 0
        filled = int(self.bar_width * consumed / total) if total > 0 else 0
        bar = "█" * filled + "░" * (self.bar_width - filled)

        # 计算速度
        elapsed = now - self.start_time
        speed_text = ""
        if elapsed > 0 and consumed > 0:
            speed = consumed / elapsed
            speed_text = f" {format_file_size(int(speed))}/s"

        print(
            f"\r{self.name}: [{bar}] {percentage:.1f}% "
            f"{format_file_size(consumed)}/{format_file_size(total)}"
            f"{speed_text}",
            end="",
            file=self.file,
        )
        self.last_update = now

        if consumed >= total:
            print(file=self.file)  # 完成时换行


class OverallProgress:
    """整体执行进度管理，避免0到100的跳跃"""

    def __init__(self, name: str = "执行进度", file: Optional[TextIO] = None):
        self.name = name
        self.file = file or sys.stderr
        self.phases = []  # 阶段列表: [(name, weight), ...]
        self.current_phase = 0
        self.phase_progress = 0
        self.progress_bar = None

    def add_phase(self, name: str, weight: int = 1):
        """添加执行阶段"""
        self.phases.append((name, weight))

    def start_phase(self, phase_index: int):
        """开始某个阶段"""
        if 0 <= phase_index < len(self.phases):
            self.current_phase = phase_index
            self.phase_progress = 0
            if self.progress_bar is None:
                sum(weight for _, weight in self.phases)
                self.progress_bar = ProgressBar(self.name, file=self.file)

    def update_phase_progress(self, progress: float):
        """更新当前阶段的进度 (0-1)"""
        self.phase_progress = min(1.0, max(0.0, progress))
        self._update_overall()

    def _update_overall(self):
        """更新整体进度"""
        if not self.phases or self.progress_bar is None:
            return

        total_progress = 0
        total_weight = sum(weight for _, weight in self.phases)

        for i, (_, weight) in enumerate(self.phases):
            if i < self.current_phase:
                total_progress += weight
            elif i == self.current_phase:
                total_progress += weight * self.phase_progress

        # 转换为百分比
        overall_consumed = int(total_progress * 100)
        overall_total = total_weight * 100

        self.progress_bar.update(overall_consumed, overall_total)

    def complete_phase(self):
        """完成当前阶段"""
        self.phase_progress = 1.0
        self._update_overall()

    def finish(self):
        """完成所有任务"""
        if self.progress_bar:
            total_weight = sum(weight for _, weight in self.phases)
            self.progress_bar.update(total_weight * 100, total_weight * 100)


def create_progress_callback(filename: Optional[str] = None, **kwargs) -> Callable:
    """创建进度回调函数（简化版，只有bar样式）"""
    progress_bar = None
    display_name = filename or "上传"

    def callback(consumed: int, total: int):
        nonlocal progress_bar
        if progress_bar is None:
            progress_bar = ProgressBar(display_name, **kwargs)
        progress_bar.update(consumed, total)

    return callback


def create_multi_file_progress_callback(
    files: list[dict[str, Any]], **kwargs
) -> Callable:
    """创建多文件上传的整体进度回调"""
    if not files:
        return lambda idx: lambda c, t: None

    # 创建整体进度管理器
    overall = OverallProgress("上传进度", **kwargs)

    # 添加文件处理阶段
    for i, file_info in enumerate(files):
        file_name = file_info.get("name", f"文件{i + 1}")
        # 根据文件大小计算权重，最小权重为1
        file_size = file_info.get("size", 1024)  # 默认1KB
        weight = max(1, file_size // (100 * 1024))  # 每100KB为1个权重单位
        overall.add_phase(file_name, weight)

    def get_file_callback(file_index: int) -> Callable[[int, int], None]:
        if 0 <= file_index < len(files):
            overall.start_phase(file_index)

        def callback(consumed: int, total: int):
            # 更新当前阶段进度
            phase_progress = consumed / total if total > 0 else 0
            overall.update_phase_progress(phase_progress)

            # 如果文件完成
            if consumed >= total:
                overall.complete_phase()

                # 如果是最后一个文件
                if file_index == len(files) - 1:
                    overall.finish()

        return callback

    return get_file_callback



"""工具函数"""

import math
import os
import re
from decimal import ROUND_HALF_UP, Decimal
from pathlib import Path
from typing import Any, Union

import orjson
import requests

from .exceptions import APIError


def sanitize_filename(filename: str, max_length: int = 100) -> str:
    """清理文件名，保留中文等 Unicode 字符

    Args:
        filename: 原始文件名
        max_length: 最大长度

    Returns:
        清理后的文件名
    """
    # 保留字母、数字、中文、日文、韩文等常见字符，以及 .-_
    filename = re.sub(
        r"[^\w\u4e00-\u9fff\u3040-\u309f\u30a0-\u30ff\uac00-\ud7af\.-]", "_", filename
    )

    # 移除开头和结尾的特殊字符
    filename = filename.strip("._")

    # 限制长度
    if len(filename) > max_length:
        name, ext = os.path.splitext(filename)
        if ext:
            name = name[: max_length - len(ext)]
            filename = name + ext
        else:
            filename = filename[:max_length]

    return filename or "unnamed"


def generate_unique_filename(base_name: str, extension: str = "") -> str:
    """生成唯一文件名，精确到毫秒

    Args:
        base_name: 基础文件名
        extension: 文件扩展名

    Returns:
        带毫秒时间戳的唯一文件名
    """
    from datetime import datetime

    # 确保扩展名格式正确
    if extension and not extension.startswith("."):
        extension = "." + extension

    # 毫秒时间戳：HHMMSS_fff（精确到毫秒）
    timestamp = datetime.now().strftime("%H%M%S_%f")[:-3]

    # 清理并截断文件名，留出时间戳空间
    clean_base = sanitize_filename(base_name)[:80]  # 最多80字符

    return f"{clean_base}_{timestamp}{extension}"


def read_file_content(file_path: Union[str, Path]) -> Union[dict[str, Any], bytes]:
    """读取文件内容

    Args:
        file_path: 文件路径

    Returns:
        JSON 文件返回字典，其他文件返回字节
    """
    file_path = Path(file_path)

    if not file_path.exists():
        raise FileNotFoundError(f"文件不存在: {file_path}")

    if file_path.suffix.lower() == ".json":
        with open(file_path, encoding="utf-8") as f:
            return orjson.loads(f.read())
    else:
        with open(file_path, "rb") as f:
            return f.read()


def get_file_size(file_path: Union[str, Path]) -> int:
    """获取文件大小（字节）

    Args:
        file_path: 文件路径

    Returns:
        文件大小（字节）
    """
    return Path(file_path).stat().st_size


def format_file_size(size_bytes: int) -> str:
    """格式化文件大小为人类可读格式

    Args:
        size_bytes: 文件大小（字节）

    Returns:
        格式化后的字符串
    """
    for unit in ["B", "KB", "MB", "GB", "TB"]:
        if size_bytes < 1024.0:
            return f"{size_bytes:.1f} {unit}"
        size_bytes /= 1024.0
    return f"{size_bytes:.1f} PB"


def handle_api_response(
    response: requests.Response, operation: str = "API调用"
) -> dict[str, Any]:
    """统一处理 API 响应

    Args:
        response: requests 响应对象
        operation: 操作描述，用于错误信息

    Returns:
        解析后的响应数据

    Raises:
        APIError: 当 HTTP 状态码非 200 或业务错误码非 200 时
    """
    if response.status_code == 200:
        response_data = orjson.loads(response.content)

        # 检查业务错误码
        if "code" in response_data and response_data.get("code") != 200:
            error_msg = f"{operation}失败: {response_data.get('message', '未知错误')}"
            raise APIError(error_msg, response_data.get("code"))

        return response_data
    else:
        error_msg = f"{operation}失败: HTTP {response.status_code}"
        try:
            error_detail = orjson.loads(response.content)
            if "detail" in error_detail:
                error_msg += f" - {error_detail['detail']}"
            elif "error" in error_detail:
                error_msg += f" - {error_detail['error']}"
        except (orjson.JSONDecodeError, ValueError, KeyError):
            error_msg += f" - {response.text}"
        except Exception as e:
            error_msg += f" - 响应解析失败: {e}"

        raise APIError(error_msg, response.status_code)


def round_floats(o: Any, precision: int = 3) -> Any:
    """递归处理数据结构中的浮点数，使用 decimal 实现精确四舍五入

    对于 NaN 和 Infinity 等特殊值，转换为 None 以确保 JSON 兼容性。

    Args:
        o: 要处理的对象
        precision: 保留的小数位数（0-10），默认为 3

    Returns:
        处理后的对象，NaN/Infinity 转换为 None
    """
    if isinstance(o, float):
        if math.isnan(o) or math.isinf(o):
            return None

        # 使用 decimal 进行精确四舍五入
        try:
            decimal_value = Decimal(str(o))
            quantizer = Decimal("0.1") ** precision
            rounded = decimal_value.quantize(quantizer, rounding=ROUND_HALF_UP)
            return float(rounded)
        except (ValueError, ArithmeticError):
            # 如果 decimal 转换失败，回退到普通 round
            return round(o, precision)
    elif isinstance(o, dict):
        return {k: round_floats(v, precision) for k, v in o.items()}
    elif isinstance(o, (list, tuple)):
        return type(o)(round_floats(x, precision) for x in o)
    return o


def get_json_size(data: Any) -> int:
    """获取数据序列化为 JSON 后的字节大小

    Args:
        data: 要序列化的数据

    Returns:
        JSON 字节大小
    """
    json_str = orjson.dumps(data).decode("utf-8")
    return len(json_str.encode("utf-8"))


def get_temp_dir(custom_dir: str = None, purpose: str = "oss-upload") -> str:
    """获取可用的临时目录

    Args:
        custom_dir: 自定义临时目录，优先使用
        purpose: 目录用途，用于创建子目录

    Returns:
        可用的临时目录路径
    """
    import tempfile

    # 候选目录列表，按优先级排序
    candidates = []
    if custom_dir and isinstance(custom_dir, str):
        candidates.append(os.path.expanduser(custom_dir))

    candidates.extend(
        [
            os.path.join(tempfile.gettempdir(), f"lims2-{purpose}"),
            os.path.expanduser(f"~/.cache/lims2-sdk/{purpose}"),
            f".lims2-{purpose}",
        ]
    )

    # 测试每个目录
    for temp_dir in candidates:
        try:
            Path(temp_dir).mkdir(parents=True, exist_ok=True)
            test_file = Path(temp_dir) / ".test"
            test_file.write_text("test")
            test_file.unlink()
            return temp_dir
        except (PermissionError, OSError):
            continue

    # 兜底方案
    fallback = os.path.join(tempfile.gettempdir(), f"lims2-{os.getpid()}")
    Path(fallback).mkdir(parents=True, exist_ok=True)
    return fallback


def clean_plotly_data(data: dict) -> dict:
    """清理Plotly数据，移除不必要的属性以减少文件大小和避免缩略图生成异常

    Args:
        data: Plotly图表数据

    Returns:
        dict: 清理后的数据
    """
    import copy

    # 需要移除的问题属性（黑名单）
    blacklist_attributes = {
        # trace级别的问题属性
        "frame",
        "frames",
        "animation",
        "animationopts",
        # layout级别的问题属性
        "sliders",
        "updatemenus",
        "template",
        "transitions",
    }

    def remove_blacklisted_attrs(obj):
        """递归移除黑名单中的属性"""
        if isinstance(obj, dict):
            cleaned = {}
            for key, value in obj.items():
                if key not in blacklist_attributes:
                    cleaned[key] = remove_blacklisted_attrs(value)
            return cleaned
        elif isinstance(obj, list):
            return [remove_blacklisted_attrs(item) for item in obj]
        else:
            return obj

    return remove_blacklisted_attrs(copy.deepcopy(data))



"""简化的网络模块

移除DNS回退策略，保留基本的网络重试功能
"""

import logging
from functools import wraps
from typing import Callable

import requests
from tenacity import (
    after_log,
    before_sleep_log,
    retry,
    retry_if_exception_type,
    stop_after_attempt,
    wait_exponential,
    wait_exponential_jitter,
)

logger = logging.getLogger(__name__)


def get_retryable_network_errors() -> tuple:
    """返回可重试的网络错误类型"""
    # 基础网络错误
    base_errors = (
        requests.exceptions.ConnectionError,
        requests.exceptions.Timeout,
        requests.exceptions.ReadTimeout,
        requests.exceptions.ConnectTimeout,
        requests.exceptions.SSLError,
        OSError,  # 包括网络相关的系统错误
    )

    # 添加OSS相关的可重试错误
    try:
        import oss2.exceptions

        oss_errors = (
            oss2.exceptions.ServerError,  # 服务器错误，通常可重试
            oss2.exceptions.RequestError,  # 请求错误的某些情况可重试
        )
        return base_errors + oss_errors
    except ImportError:
        return base_errors


def network_retry(
    max_retries: int = 3,
    base_delay: float = 1.0,
    max_delay: float = 60.0,
    backoff_factor: float = 2.0,
    jitter: bool = True,
) -> Callable:
    """网络重试装饰器

    Args:
        max_retries: 最大重试次数
        base_delay: 基础延迟时间（秒）
        max_delay: 最大延迟时间（秒）
        backoff_factor: 退避因子
        jitter: 是否添加抖动

    Returns:
        装饰器函数
    """
    # 配置等待策略
    if jitter:
        # 使用指数退避加抖动（Google推荐的策略）
        wait_strategy = wait_exponential_jitter(
            initial=base_delay, max=max_delay, jitter=base_delay
        )
    else:
        # 仅使用指数退避
        wait_strategy = wait_exponential(
            multiplier=base_delay, max=max_delay, exp_base=backoff_factor
        )

    def decorator(func):
        @wraps(func)
        @retry(
            retry=retry_if_exception_type(get_retryable_network_errors()),
            stop=stop_after_attempt(max_retries + 1),  # +1 包含首次尝试
            wait=wait_strategy,
            before_sleep=before_sleep_log(logger, logging.DEBUG),
            after=after_log(logger, logging.DEBUG),
            reraise=True,
        )
        def wrapper(*args, **kwargs):
            return func(*args, **kwargs)

        return wrapper

    return decorator



"""配置管理模块"""

import os
from typing import Optional


class Config:
    """配置管理类"""

    def __init__(self, api_url: Optional[str] = None, token: Optional[str] = None):
        """初始化配置

        Args:
            api_url: API 地址，如果不提供则从环境变量读取
            token: API Token，如果不提供则从环境变量读取
        """
        self.api_url = (
            api_url
            if api_url is not None
            else os.environ.get("LIMS2_API_URL") or "https://api-v1.lims2.com"
        )
        self.token = token or os.environ.get("LIMS2_API_TOKEN")
        self.team_id = (
            os.environ.get("LIMS2_TEAM_ID") or "be4e0714c336d2b4bfe00718310d01d5"
        )

        # 网络配置
        self.timeout = int(os.environ.get("LIMS2_TIMEOUT", "600"))  # 默认10分钟
        self.connection_timeout = int(
            os.environ.get("LIMS2_CONNECTION_TIMEOUT", "30")
        )  # 连接超时
        self.read_timeout = int(os.environ.get("LIMS2_READ_TIMEOUT", "300"))  # 读取超时

        # OSS配置
        self.oss_endpoint = (
            os.environ.get("LIMS2_OSS_ENDPOINT")
            or "https://oss-cn-shanghai.aliyuncs.com"
        )
        self.oss_bucket_name = os.environ.get("LIMS2_OSS_BUCKET_NAME") or "protree"

        # SSL验证配置 - 默认禁用以适应内网环境
        self.verify_ssl = os.environ.get("LIMS2_VERIFY_SSL", "false").lower() != "false"

        # 临时目录配置
        self.custom_temp_dir = os.environ.get("LIMS2_TEMP_DIR")  # 自定义临时目录

        # 缩略图配置
        self.auto_generate_thumbnail = self._get_bool_env(
            "LIMS2_AUTO_GENERATE_THUMBNAIL", True
        )
        self.thumbnail_width = int(os.environ.get("LIMS2_THUMBNAIL_WIDTH", "800"))
        self.thumbnail_height = int(os.environ.get("LIMS2_THUMBNAIL_HEIGHT", "600"))
        self.thumbnail_format = os.environ.get("LIMS2_THUMBNAIL_FORMAT", "webp")

    def validate(self) -> None:
        """验证配置是否完整"""
        if not self.api_url:
            raise ValueError("API URL 未配置，请设置环境变量 LIMS2_API_URL")
        if not self.token:
            raise ValueError("API Token 未配置，请设置环境变量 LIMS2_API_TOKEN")

    def get_headers(self) -> dict[str, str]:
        """获取 API 请求头"""
        return {
            "Authorization": f"Bearer {self.token}",
            "Content-Type": "application/json",
        }

    def _get_bool_env(self, key: str, default: bool) -> bool:
        """从环境变量获取布尔值"""
        value = os.environ.get(key)
        if value is None:
            return default
        return value.lower() in ("true", "1", "yes", "on")



"""缩略图生成模块"""

from typing import Any, Optional

import plotly.graph_objects as go
import plotly.io as pio


def generate_thumbnail(
    chart_data: dict[str, Any],
    width: int = 800,
    height: int = 600,
    format: str = "webp",
) -> Optional[bytes]:
    """生成Plotly图表的缩略图

    Args:
        chart_data: 标准的Plotly图表数据，应包含'data'和'layout'字段
        width: 图片宽度（默认800像素）
        height: 图片高度（默认600像素）
        format: 图片格式，支持'webp'、'png'、'jpeg'等（默认'webp'）

    Returns:
        bytes: 图片的字节数据，如果生成失败返回None
    """
    try:
        if not isinstance(chart_data, dict) or "data" not in chart_data:
            return None

        # 使用 skip_invalid=True 跳过无效属性，validate=False 禁用验证
        fig = go.Figure(chart_data, skip_invalid=True)
        return pio.to_image(
            fig, format=format, width=width, height=height, validate=False
        )

    except Exception:
        return None



"""主客户端模块"""

from typing import Optional

import requests
import urllib3

from .chart import ChartService
from .config import Config
from .exceptions import ConfigError
from .storage import StorageService

# 禁用 urllib3 的 SSL 警告，因为我们在内网环境中使用，用户可以根据需要启用 SSL 验证
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


class Lims2Client:
    """Lims2 SDK 主客户端

    提供图表和文件存储等服务的统一入口。典型用法：

    Example:
        >>> from lims2 import Lims2Client
        >>> client = Lims2Client()
        >>>
        >>> # 上传图表
        >>> client.chart.upload("plot.json", project_id="proj_001")
        >>>
        >>> # 上传文件
        >>> client.storage.upload_file("results.csv", project_id="proj_001")
    """

    def __init__(self, api_url: Optional[str] = None, token: Optional[str] = None):
        """初始化客户端

        Args:
            api_url: API 地址（可选，默认从环境变量读取）
            token: API Token（可选，默认从环境变量读取）
        """
        # 初始化配置
        self.config = Config(api_url, token)

        # 验证配置
        try:
            self.config.validate()
        except ValueError as e:
            raise ConfigError(str(e))

        # 创建标准的 HTTP 会话
        self.session = requests.Session()
        self.session.headers.update(self.config.get_headers())

        # 配置SSL验证
        self.session.verify = self.config.verify_ssl

        # 直接初始化服务
        self.chart = ChartService(self)
        self.storage = StorageService(self)

    def close(self) -> None:
        """关闭客户端，清理资源"""
        if hasattr(self, "session"):
            self.session.close()

    def __enter__(self):
        """上下文管理器入口"""
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """上下文管理器出口"""
        self.close()

    def __repr__(self) -> str:
        return f"Lims2Client(api_url={self.config.api_url!r})"



"""图表服务模块

基于原 biotree_chart 功能实现
"""

import gzip
from pathlib import Path
from typing import Any, Optional, Union

import orjson

from .network import network_retry
from .oss_base import OSSMixin
from .thumbnail import generate_thumbnail
from .utils import (
    clean_plotly_data,
    generate_unique_filename,
    get_file_size,
    handle_api_response,
    read_file_content,
    round_floats,
)


class ChartService(OSSMixin):
    """图表服务"""

    def __init__(self, client):
        """初始化图表服务

        Args:
            client: Lims2Client 实例
        """
        self.client = client
        self.config = client.config
        self.session = client.session

        # 初始化OSS混入功能
        self.__init_oss__()

    def upload(
        self,
        data_source: Union[dict[str, Any], str, Path],
        project_id: str,
        chart_name: str,
        sample_id: Optional[str] = None,
        chart_type: Optional[str] = None,
        description: Optional[str] = None,
        contrast: Optional[str] = None,
        analysis_node: Optional[str] = None,
        precision: Optional[int] = None,
        generate_thumbnail: Optional[bool] = None,
    ) -> dict[str, Any]:
        """上传图表

        Args:
            data_source: 图表数据源，可以是字典、文件路径或 Path 对象
            project_id: 项目 ID
            chart_name: 图表名称
            sample_id: 样本 ID（可选）
            chart_type: 图表类型（可选）
            description: 图表描述（可选）
            contrast: 对比策略（可选）
            analysis_node: 分析节点名称（可选）
            precision: 浮点数精度控制，保留小数位数（0-10，默认3）
            generate_thumbnail: 是否生成缩略图，None时使用配置中的auto_generate_thumbnail

        Returns:
            上传结果
        """
        # 参数验证
        if not chart_name:
            raise ValueError("图表名称不能为空")
        if not project_id:
            raise ValueError("项目 ID 不能为空")
        if not data_source:
            raise ValueError("数据源不能为空")
        if precision is not None and not 0 <= precision <= 10:
            raise ValueError("precision 必须在 0-10 之间")

        # 构建请求数据
        request_data = {
            "chart_name": chart_name,
            "project_id": project_id,
            "chart_type": chart_type,
            "description": description,
        }

        # 添加可选参数
        if sample_id:
            request_data["sample_id"] = sample_id
        if contrast:
            request_data["contrast"] = contrast
        if analysis_node:
            request_data["analysis_node"] = analysis_node

        # 根据数据源类型处理
        if isinstance(data_source, dict):
            return self._upload_from_dict(
                request_data, data_source, precision, generate_thumbnail
            )
        elif isinstance(data_source, (str, Path)):
            return self._upload_from_file(
                request_data, data_source, precision, generate_thumbnail
            )
        else:
            raise ValueError("数据源必须是字典、文件路径或 Path 对象")

    def _upload_from_dict(
        self,
        request_data: dict[str, Any],
        chart_data: dict[str, Any],
        precision: Optional[int] = None,
        generate_thumbnail: Optional[bool] = None,
    ) -> dict[str, Any]:
        """从字典数据上传图表

        关键字段说明:
        - file_name: 不带扩展名的文件名，用于服务端显示
        - file_format: 文件格式，如'json'，用于服务端验证
        - oss_key: OSS中的存储路径，包含带时间戳的唯一文件名
        """
        # 检测渲染器类型
        if "data" in chart_data and "layout" in chart_data:
            request_data["renderer_type"] = "plotly"
        elif "elements" in chart_data or (
            "nodes" in chart_data and "edges" in chart_data
        ):
            request_data["renderer_type"] = "cytoscape"
        else:
            raise ValueError("不支持的图表数据格式")

        # 应用精度控制（默认使用 3 位小数）
        if precision is None:
            precision = 3  # 默认精度为 3

        # 清理Plotly数据，移除不必要的属性
        if request_data["renderer_type"] == "plotly":
            chart_data = clean_plotly_data(chart_data)

        # 应用精度控制并序列化
        chart_data = round_floats(chart_data, precision)
        json_str = orjson.dumps(chart_data).decode("utf-8")

        # 设置文件信息
        # 从文件上传时已设置file_name，从字典上传时使用chart_name
        if "file_name" not in request_data:
            request_data["file_name"] = request_data["chart_name"]
        request_data["file_format"] = "json"

        # 压缩并上传到OSS
        compressed_data = gzip.compress(json_str.encode("utf-8"))
        filename = generate_unique_filename(
            request_data["file_name"], "json"
        )  # 用于OSS存储
        oss_key = self._build_chart_oss_key(
            request_data["project_id"], request_data.get("sample_id"), filename
        )
        bucket = self._get_oss_bucket(request_data["project_id"])

        # 上传到OSS，如果失败则直接抛出原始异常，不创建数据库记录
        self._put_object_with_retry(
            bucket,
            oss_key,
            compressed_data,
            headers={
                "Content-Type": "application/json",
                "Content-Encoding": "gzip",
            },
        )

        request_data["oss_key"] = oss_key

        # 生成缩略图
        should_generate = (
            generate_thumbnail
            if generate_thumbnail is not None
            else self.config.auto_generate_thumbnail
        )
        if should_generate and request_data.get("renderer_type") == "plotly":
            print("开始生成缩略图...")
            self._generate_and_upload_thumbnail(chart_data, request_data)

        # 创建图表记录
        return self._create_chart_record(request_data)

    def _upload_from_file(  # noqa: C901
        self,
        request_data: dict[str, Any],
        file_path: Union[str, Path],
        precision: Optional[int] = None,
        generate_thumbnail: Optional[bool] = None,
    ) -> dict[str, Any]:
        """从文件上传图表"""
        file_path = Path(file_path)

        if not file_path.exists():
            raise FileNotFoundError(f"文件不存在: {file_path}")

        file_format = file_path.suffix.lower().strip(".")
        request_data["file_size"] = get_file_size(file_path)

        # JSON 文件特殊处理
        if file_format == "json":
            try:
                chart_data = read_file_content(file_path)
                if isinstance(chart_data, dict):
                    # 使用原始文件名（不带扩展名）
                    request_data["file_name"] = file_path.stem
                    return self._upload_from_dict(
                        request_data, chart_data, precision, generate_thumbnail
                    )
            except FileNotFoundError:
                raise
            except orjson.JSONDecodeError as e:
                raise ValueError(f"JSON 文件格式错误: {e}")
            except Exception as e:
                raise ValueError(f"读取 JSON 文件失败: {e}")

        # 其他文件类型
        if file_format in ["png", "jpg", "jpeg", "svg", "pdf"]:
            request_data["renderer_type"] = "image"
        elif file_format == "html":
            request_data["renderer_type"] = "html"
        else:
            raise ValueError(f"不支持的文件格式: {file_format}")

        # 设置文件信息
        request_data["file_format"] = file_format
        request_data["file_name"] = file_path.stem  # 不带扩展名
        filename = generate_unique_filename(file_path.stem, file_format)  # 用于OSS存储

        # 构建OSS键名
        oss_key = self._build_chart_oss_key(
            request_data["project_id"], request_data.get("sample_id"), filename
        )

        # 读取文件内容
        file_content = read_file_content(file_path)
        if isinstance(file_content, dict):
            file_content = orjson.dumps(file_content)

        # 上传到 OSS
        content_type = self._get_content_type(file_format)
        bucket = self._get_oss_bucket(request_data["project_id"])

        # 上传到OSS，如果失败则直接抛出原始异常，不创建数据库记录
        self._put_object_with_retry(
            bucket, oss_key, file_content, headers={"Content-Type": content_type}
        )

        request_data["oss_key"] = oss_key

        # 创建图表记录
        return self._create_chart_record(request_data)

    def _build_chart_oss_key(
        self, project_id: str, sample_id: Optional[str], filename: str
    ) -> str:
        """构建图表的OSS键名

        Args:
            project_id: 项目ID
            sample_id: 样本ID（可选）
            filename: 文件名

        Returns:
            str: OSS键名，格式为biochart/{env}/project_id/[sample_id/]filename
            - 环境前缀：生产环境使用media，测试环境使用test
        """
        # 使用环境相关的路径前缀
        env_prefix = self._get_oss_path_prefix()
        parts = ["biochart", env_prefix, project_id]
        if sample_id:
            parts.append(sample_id)
        parts.append(filename)
        return "/".join(parts)

    @network_retry(max_retries=3, base_delay=1.0, max_delay=15.0)
    def _create_chart_record(self, request_data: dict[str, Any]) -> dict[str, Any]:
        """创建图表记录"""

        request_data["token"] = self.config.token
        request_data["team_id"] = self.config.team_id

        # 使用配置的超时时间和分离的连接/读取超时
        timeout = (self.config.connection_timeout, self.config.read_timeout)

        response = self.session.post(
            f"{self.config.api_url}/get_data/biochart/create_chart/",
            json=request_data,
            timeout=timeout,
        )
        return handle_api_response(response, "创建图表记录")

    def _get_content_type(self, file_format: str) -> str:
        """获取文件内容类型"""
        content_types = {
            "json": "application/json",
            "png": "image/png",
            "jpg": "image/jpeg",
            "jpeg": "image/jpeg",
            "svg": "image/svg+xml",
            "pdf": "application/pdf",
            "html": "text/html",
        }
        return content_types.get(file_format, "application/octet-stream")

    def _generate_and_upload_thumbnail(
        self, chart_data: dict[str, Any], request_data: dict[str, Any]
    ) -> None:
        """生成并上传缩略图到OSS"""
        try:
            # 此时chart_data已经被清理过了，直接使用
            thumb_bytes = generate_thumbnail(
                chart_data,
                self.config.thumbnail_width,
                self.config.thumbnail_height,
                self.config.thumbnail_format,
            )
            if thumb_bytes:
                # file_name已经不带扩展名，直接使用
                thumb_filename = (
                    f"{request_data['file_name']}_thumb.{self.config.thumbnail_format}"
                )
                thumb_oss_key = self._build_chart_oss_key(
                    request_data["project_id"],
                    request_data.get("sample_id"),
                    thumb_filename,
                )
                bucket = self._get_oss_bucket(request_data["project_id"])
                content_type = f"image/{self.config.thumbnail_format}"

                self._put_object_with_retry(
                    bucket,
                    thumb_oss_key,
                    thumb_bytes,
                    headers={"Content-Type": content_type},
                )
                img_url = f"https://image.lims2.com/{thumb_oss_key}"
                request_data["img_url"] = img_url
                print(f"缩略图已生成: {img_url}")
        except Exception:
            pass  # 缩略图失败不影响正常上传
